import java.util.*;
import java.io.*;

public class Client {
    public static void main(String[] args) {
        System.out.println("| n | set | ArrayList | LinkedList | BinaryTree | BinarySearchTree |");
        
        int[] setSizes = {10, 100, 1000, 10000, 100000};
        for (int i = 0; i < setSizes.length; i++) {
            long[] times = addTest(setSizes[i]);
            System.out.println(Arrays.toString(times));           
        }
        
        csvData();
    }
    
    // generate n random numbers and time how long it takes
    // to add that set of data into each of the data structures
    // we've learned about this quarter
    public static long[] addTest(int n) {
        long[] times = new long[6];
        times[0] = n;
        long start = System.nanoTime();
        int[] dataSet = randomDataSet(n);
        long finish = System.nanoTime();
        times[1] = finish - start;

        ArrayIntList arrList = new ArrayIntList();
        start = System.nanoTime();
        for (int i = 0; i < n; i++) {
            arrList.add(dataSet[i]);
        }
        finish = System.nanoTime();
        times[2] = finish - start;

        LinkedIntList linkList = new LinkedIntList();
        start = System.nanoTime();
        for (int i = 0; i < n; i++) {
            linkList.add(dataSet[i]);
        }
        finish = System.nanoTime();
        times[3] = finish - start;

        IntTree binaryTree = new IntTree();
        start = System.nanoTime();
        for (int i = 0; i < n; i++) {
            binaryTree.add(dataSet[i]);
        }
        finish = System.nanoTime();
        times[4] = finish - start;

        IntSearchTree bst = new IntSearchTree();
        start = System.nanoTime();
        for (int i = 0; i < n; i++) {
            bst.add(dataSet[i]);
        }
        finish = System.nanoTime();
        times[5] = finish - start; 

        return times;                   
    }

    // generate n random numbers and time how long it takes
    // to add that set of data into each of the data structures
    // we've learned about this quarter. Runs trials trials and 
    // throws away the results for the first discard trials and 
    // reports an average for each data structure
    public static double[] addMultipleTests(int n, int trials, int discard) {
        long[] totalTimes = new long[6];
        for (int i = 0; i < trials; i++) {
            long[] times = addTest(n);

            // Add to total if not discarding
            if (i >= discard) {
                for (int j = 0; j < times.length; j++) {
                    totalTimes[j] += times[j];
                }
            }
        }

        // Compute average
        double[] averageTimes = new double[6];
        for (int i = 0; i < totalTimes.length; i++) {
            averageTimes[i] = (double) totalTimes[i] / (trials - discard);
        }
        return averageTimes;
    }

    // generates a set of n random numbers between 0 and max int
    public static int[] randomDataSet(int n) {
        Random rand = new Random();
        int[] dataSet = new int[n];
        for (int i = 0; i < n; i++) {
            dataSet[i] = rand.nextInt();
        }
        return dataSet;
    }

    // generates a large quantity of timing data and creates a CSV file
    // you can use this to copy and paste the values into a spreadsheet
    // to graph the runtimes
    public static void csvData() {
        try {
            File resultsFile = new File("timeTest.csv");
            PrintWriter results = new PrintWriter(resultsFile);
            for (int i = 0; i <= 10000; i+=100) {
                double[] averageTimes = addMultipleTests(i, 25, 5);
                results.print(averageTimes[0]);
                for (int j = 1; j < averageTimes.length; j++) {
                    results.print(", " + averageTimes[j]);
                }
                results.println();  
            }
            results.close();         
        } catch (IOException e) {
            System.out.println("file error");
        }
    }
}
